@@ -1,3 +1,4 @@
+from dataclasses import InitVar
 from typing import Any, ClassVar, Generic, List, Optional, TypeVar, Union
 
 from typing_extensions import Self
@@ -25,6 +26,7 @@ SelfReferencingModel.model_rebuild()
 
 model = Model(x=1, y='y')
 Model(x=1, y='y', z='z')
+# MYPY: error: Unexpected keyword argument "z" for "Model"  [call-arg]
 model.x = 2
 model.model_validate(model)
 
@@ -38,6 +40,7 @@ class KwargsModel(BaseModel, from_attributes=True):
 
 kwargs_model = KwargsModel(x=1, y='y')
 KwargsModel(x=1, y='y', z='z')
+# MYPY: error: Unexpected keyword argument "z" for "KwargsModel"  [call-arg]
 kwargs_model.x = 2
 kwargs_model.model_validate(kwargs_model.__dict__)
 
@@ -83,10 +86,12 @@ class KwargsNoMutationModel(BaseModel, frozen=True):
 
 
 class KwargsMutationModel(KwargsNoMutationModel, frozen=False, from_attributes=True):
+# MYPY: error: Cannot inherit non-frozen dataclass from a frozen one  [misc]
     a: int = 1
 
 
 KwargsMutationModel(x=1).x = 2
+# MYPY: error: Property "x" defined in "KwargsNoMutationModel" is read-only  [misc]
 KwargsMutationModel.model_validate(model.__dict__)
 
 
@@ -133,9 +138,6 @@ class AddProject:
 
 
 p = AddProject(name='x', slug='y', description='z')
-# MYPY: error: Unexpected keyword argument "name" for "AddProject"  [call-arg]
-# MYPY: error: Unexpected keyword argument "slug" for "AddProject"  [call-arg]
-# MYPY: error: Unexpected keyword argument "description" for "AddProject"  [call-arg]
 
 
 class TypeAliasAsAttribute(BaseModel):
@@ -260,26 +262,13 @@ class OrmMixin(Generic[_TModel, _TType]):
         return cls.from_orm(model)
 
 
-import sys  # noqa E402
-
-if sys.version_info >= (3, 8):
-    from dataclasses import InitVar  # E402
-
-    InitVarStr = InitVar[str]
-else:
-    # InitVar is not supported in 3.7 due to loss of type information
-    InitVarStr = str
-
-
 @dataclass
 class MyDataClass:
-    foo: InitVarStr
+    foo: InitVar[str]
     bar: str
 
 
 MyDataClass(foo='foo', bar='bar')
-# MYPY: error: Unexpected keyword argument "foo" for "MyDataClass"  [call-arg]
-# MYPY: error: Unexpected keyword argument "bar" for "MyDataClass"  [call-arg]
 
 
 def get_my_custom_validator(field_name: str) -> Any:
@@ -294,7 +283,7 @@ def foo() -> None:
     class MyModel(BaseModel):
         number: int
         custom_validator = get_my_custom_validator('number')  # type: ignore[pydantic-field]
-# MYPY: error: Unused "type: ignore" comment
+# MYPY: error: Unused "type: ignore" comment  [unused-ignore]
 
         @model_validator(mode='before')
         @classmethod
@@ -306,3 +295,17 @@ def foo() -> None:
             return self
 
     MyModel(number=2)
+
+
+class InnerModel(BaseModel):
+    my_var: Union[str, None] = Field(default=None)
+
+
+class OuterModel(InnerModel):
+    pass
+
+
+m = OuterModel()
+if m.my_var is None:
+    # In https://github.com/pydantic/pydantic/issues/7399, this was unreachable
+    print('not unreachable')
