@@ -69,6 +69,13 @@ cases = (
         'success.py',
         pytest.mark.skipif(MYPY_VERSION_TUPLE > (1, 0, 1), reason='Need to handle some more things for mypy >=1.1.1'),
     ).build()
+    + MypyCasesBuilder(
+        ['mypy-default.ini', 'pyproject-default.toml'],
+        'root_models.py',
+        pytest.mark.skipif(
+            MYPY_VERSION_TUPLE < (1, 1, 1), reason='`dataclass_transform` only supported on mypy >= 1.1.1'
+        ),
+    ).build()
     + MypyCasesBuilder(
         'mypy-default.ini', ['plugin_success.py', 'plugin_success_baseConfig.py', 'metaclass_args.py']
     ).build()
@@ -102,14 +109,8 @@ cases = (
         ('mypy-plugin-strict.ini', 'plugin_default_factory.py'),
         ('mypy-plugin-strict-no-any.ini', 'dataclass_no_any.py'),
         ('mypy-plugin-very-strict.ini', 'metaclass_args.py'),
-        pytest.param(
-            'pyproject-default.toml',
-            'computed_fields.py',
-            marks=pytest.mark.skipif(
-                sys.version_info < (3, 8),
-                reason='cached_property is only available in Python 3.8+, errors are different with mypy 0.971',
-            ),
-        ),
+        ('pyproject-default.toml', 'computed_fields.py'),
+        ('pyproject-plugin-no-strict-optional.toml', 'no_strict_optional.py'),
     ]
 )
 
@@ -155,6 +156,7 @@ def get_test_config(module_path: Path, config_path: Path) -> MypyTestConfig:
     return MypyTestConfig(existing, current)
 
 
+@pytest.mark.filterwarnings('ignore:ast.:DeprecationWarning')  # these are produced by mypy in python 3.12
 @pytest.mark.parametrize('config_filename,python_filename', cases)
 def test_mypy_results(config_filename: str, python_filename: str, request: pytest.FixtureRequest) -> None:
     input_path = PYDANTIC_ROOT / 'tests/mypy/modules' / python_filename
