@@ -1,7 +1,6 @@
 import asyncio
 import inspect
 import re
-import sys
 from datetime import datetime, timezone
 from functools import partial
 from typing import Any, List, Tuple
@@ -10,12 +9,9 @@ import pytest
 from pydantic_core import ArgsKwargs
 from typing_extensions import Annotated, TypedDict
 
-from pydantic import Field, TypeAdapter, ValidationError, validate_call
+from pydantic import Field, PydanticInvalidForJsonSchema, TypeAdapter, ValidationError, validate_call
 from pydantic.main import BaseModel
 
-skip_pre_38 = pytest.mark.skipif(sys.version_info < (3, 8), reason='testing >= 3.8 behaviour only')
-skip_pre_39 = pytest.mark.skipif(sys.version_info < (3, 9), reason='testing >= 3.9 behaviour only')
-
 
 def test_args():
     @validate_call
@@ -94,9 +90,7 @@ def test_wrap():
     assert foo_bar.__name__ == 'foo_bar'
     assert foo_bar.__module__ == 'tests.test_validate_call'
     assert foo_bar.__qualname__ == 'test_wrap.<locals>.foo_bar'
-    assert isinstance(foo_bar.__pydantic_core_schema__, dict)
     assert callable(foo_bar.raw_function)
-    assert repr(foo_bar) == f'ValidateCallWrapper({repr(foo_bar.raw_function)})'
     assert repr(inspect.signature(foo_bar)) == '<Signature (a: int, b: int)>'
 
 
@@ -186,7 +180,6 @@ def test_annotated_field_can_provide_factory() -> None:
     assert foo2(1) == 100
 
 
-@skip_pre_38
 def test_positional_only(create_module):
     module = create_module(
         # language=Python
@@ -371,12 +364,11 @@ def test_item_method():
 
     # insert_assert(exc_info.value.errors(include_url=False))
     assert exc_info.value.errors(include_url=False) == [
-        {'type': 'missing_argument', 'loc': ('a',), 'msg': 'Missing required argument', 'input': ArgsKwargs(())},
-        {'type': 'missing_argument', 'loc': ('b',), 'msg': 'Missing required argument', 'input': ArgsKwargs(())},
+        {'type': 'missing_argument', 'loc': ('a',), 'msg': 'Missing required argument', 'input': ArgsKwargs((x,))},
+        {'type': 'missing_argument', 'loc': ('b',), 'msg': 'Missing required argument', 'input': ArgsKwargs((x,))},
     ]
 
 
-@skip_pre_39
 def test_class_method():
     class X:
         @classmethod
@@ -394,8 +386,8 @@ def test_class_method():
 
     # insert_assert(exc_info.value.errors(include_url=False))
     assert exc_info.value.errors(include_url=False) == [
-        {'type': 'missing_argument', 'loc': ('a',), 'msg': 'Missing required argument', 'input': ArgsKwargs(())},
-        {'type': 'missing_argument', 'loc': ('b',), 'msg': 'Missing required argument', 'input': ArgsKwargs(())},
+        {'type': 'missing_argument', 'loc': ('a',), 'msg': 'Missing required argument', 'input': ArgsKwargs((X,))},
+        {'type': 'missing_argument', 'loc': ('b',), 'msg': 'Missing required argument', 'input': ArgsKwargs((X,))},
     ]
 
 
@@ -414,32 +406,30 @@ def test_json_schema():
         'additionalProperties': False,
     }
 
-    # TODO: Uncomment when support for 3.7 is dropped.
-    # @validate_call
-    # def foo(a: int, /, b: int):
-    #     return f'{a}, {b}'
+    @validate_call
+    def foo(a: int, /, b: int):
+        return f'{a}, {b}'
 
-    # assert foo(1, 2) == '1, 2'
-    # assert TypeAdapter(foo).json_schema() == {
-    #     'maxItems': 2,
-    #     'minItems': 2,
-    #     'prefixItems': [{'title': 'A', 'type': 'integer'}, {'title': 'B', 'type': 'integer'}],
-    #     'type': 'array',
-    # }
+    assert foo(1, 2) == '1, 2'
+    assert TypeAdapter(foo).json_schema() == {
+        'maxItems': 2,
+        'minItems': 2,
+        'prefixItems': [{'title': 'A', 'type': 'integer'}, {'title': 'B', 'type': 'integer'}],
+        'type': 'array',
+    }
 
-    # @validate_call
-    # def foo(a: int, /, *, b: int, c: int):
-    #     return f'{a}, {b}, {c}'
+    @validate_call
+    def foo(a: int, /, *, b: int, c: int):
+        return f'{a}, {b}, {c}'
 
-    # assert foo(1, b=2, c=3) == '1, 2, 3'
-    # with pytest.raises(
-    #     PydanticInvalidForJsonSchema,
-    #     match=(
-    #       'Unable to generate JSON schema for arguments validator '
-    #       'with positional-only and keyword-only arguments'
-    #     ),
-    # ):
-    #     TypeAdapter(foo).json_schema()
+    assert foo(1, b=2, c=3) == '1, 2, 3'
+    with pytest.raises(
+        PydanticInvalidForJsonSchema,
+        match=(
+            'Unable to generate JSON schema for arguments validator ' 'with positional-only and keyword-only arguments'
+        ),
+    ):
+        TypeAdapter(foo).json_schema()
 
     @validate_call
     def foo(*numbers: int) -> int:
@@ -571,7 +561,6 @@ def test_validate_all():
     assert foo(0) == datetime(1970, 1, 1, tzinfo=timezone.utc)
 
 
-@skip_pre_38
 def test_validate_all_positional(create_module):
     module = create_module(
         # language=Python
@@ -610,7 +599,6 @@ def test_validator_init():
         Foo(1, 'x')
 
 
-@skip_pre_38
 def test_positional_and_keyword_with_same_name(create_module):
     module = create_module(
         # language=Python
@@ -690,6 +678,18 @@ def test_basemodel_method():
     assert bar.test('1') == (bar, 1)
 
 
+def test_dynamic_method_decoration():
+    class Foo:
+        def bar(self, value: str) -> str:
+            return f'bar-{value}'
+
+    Foo.bar = validate_call(Foo.bar)
+    assert Foo.bar
+
+    foo = Foo()
+    assert foo.bar('test') == 'bar-test'
+
+
 @pytest.mark.parametrize('decorator', [staticmethod, classmethod])
 def test_classmethod_order_error(decorator):
     name = decorator.__name__
@@ -754,3 +754,39 @@ def test_validate_call_with_slots() -> None:
     assert c.some_instance_method == c.some_instance_method
     assert c.some_class_method == c.some_class_method
     assert c.some_static_method == c.some_static_method
+
+
+def test_eval_type_backport():
+    @validate_call
+    def foo(bar: 'list[int | str]') -> 'list[int | str]':
+        return bar
+
+    assert foo([1, '2']) == [1, '2']
+    with pytest.raises(ValidationError) as exc_info:
+        foo('not a list')  # type: ignore
+    # insert_assert(exc_info.value.errors(include_url=False))
+    assert exc_info.value.errors(include_url=False) == [
+        {
+            'type': 'list_type',
+            'loc': (0,),
+            'msg': 'Input should be a valid list',
+            'input': 'not a list',
+        }
+    ]
+    with pytest.raises(ValidationError) as exc_info:
+        foo([{'not a str or int'}])  # type: ignore
+    # insert_assert(exc_info.value.errors(include_url=False))
+    assert exc_info.value.errors(include_url=False) == [
+        {
+            'type': 'int_type',
+            'loc': (0, 0, 'int'),
+            'msg': 'Input should be a valid integer',
+            'input': {'not a str or int'},
+        },
+        {
+            'type': 'string_type',
+            'loc': (0, 0, 'str'),
+            'msg': 'Input should be a valid string',
+            'input': {'not a str or int'},
+        },
+    ]
